/*
 * source/Vector/VectMath.c
 * SpacePigs VectorMania Vector math routines (Cord3d)
 *
 * history:
 * 1991-04-15: Math work begun
 * 1991-04-27: basic rotation routines tested and ready. 
 *             perspective tested and ready.
 */
#include <math.h>

#include "other/debug.h"
#include "vector/vectmath.h"

short per_a = PERSPECTIVEZOFFSET;
long per_b = PERSPECTIVEZFACTOR;
short per_vdist = VDIST;

void rotate(VECTINT x, VECTINT y, VECTINT *x1, VECTINT *y1, short si, short co)
{
    *x1 = IMULIS(x, co) - IMULIS(y, si);
    *y1 = IMULIS(x, si) + IMULIS(y, co);
}

void rotx(CORD3D *source, CORD3D *dest, int angle)
{
    short si, co;

    getsincos(angle, &si, &co);
    dest->x = source->x;
    rotate(source->z, source->y, &dest->z, &dest->y, si, co);
}
void roty(CORD3D *source, CORD3D *dest, int angle)
{
    short si, co;

    getsincos(angle, &si, &co);
    dest->y = source->y;
    rotate(source->x, source->z, &dest->x, &dest->z, si, co);
}
void rotz(CORD3D *source, CORD3D *dest, int angle)
{
    short si, co;

    getsincos(angle, &si, &co);
    dest->z = source->z;
    rotate(source->x, source->y, &dest->x, &dest->y, si, co);
}
void rotxc(CORD3D *source, CORD3D *dest, short si, short co)
{
    dest->x = source->x;
    rotate(source->z, source->y, &dest->z, &dest->y, si, co);
}
void rotyc(CORD3D *source, CORD3D *dest, short si, short co)
{
    dest->y = source->y;
    rotate(source->x, source->z, &dest->x, &dest->z, si, co);
}
void rotzc(CORD3D *source, CORD3D *dest, short si, short co)
{
    dest->z = source->z;
    rotate(source->x, source->y, &dest->x, &dest->y, si, co);
}

void perspective(CORD3D *source, CORD3D *dest)
{
    VECTINT x, y, z;
    short vdist = per_vdist;

    x = source->x;
    y = source->y;
    z = source->z + vdist;

    dest->x = (x * vdist) / z;
    dest->y = (y * vdist) / z;
    dest->z = per_a + per_b / z;
    dest->normz = z;
}

void rotcords(CORD3D *source, CORD3D *dest, int antal, AXEL axel, int angle)
{
    short si, co;
    int cord;

    getsincos(angle, &si, &co);

    for (cord = 0; cord < antal; cord++, source++, dest++) {
	VECTINT x, y, z;
	VECTINT x1, y1;
	switch (axel) {
	case X_AXEL:
	    x = source->z;
	    y = source->y;
	    z = source->x;
	    break;
	case Y_AXEL:
	    x = source->x;
	    y = source->z;
	    z = source->y;
	    break;
	case Z_AXEL:
	    x = source->x;
	    y = source->y;
	    z = source->z;
	    break;
	}
	x1 = STOI(x * co - y * si);
	y1 = STOI(x * si + y * co);
	switch (axel) {
	case X_AXEL:
	    dest->x = z;
	    dest->y = y1;
	    dest->z = x1;
	    break;
	case Y_AXEL:
	    dest->x = x1;
	    dest->y = z;
	    dest->z = y1;
	    break;
	case Z_AXEL:
	    dest->x = x1;
	    dest->y = y1;
	    dest->z = z;
	    break;
	}
    }
}

void perspectivecords(CORD3D *source, CORD3D *dest, int antal)
{
    int cord;
    short a = per_a;
    long b = per_b;
    short vdist = per_vdist;
    for (cord = 0; cord < antal; cord++, source++, dest++) {
	VECTINT x, y, z;

	x = source->x;
	y = source->y;
	z = source->z + vdist;

	dest->x = (x * vdist) / z;
	dest->y = (y * vdist) / z;
	dest->z = a + b / z;
	dest->normz = z;
    }
}

void copycords(CORD3D *source, CORD3D *dest, int antal)
{
    int cord;
    for (cord = 0; cord < antal; cord++) {
	*dest++ = *source++;
    }
}

void scalecords(CORD3D *source, CORD3D *dest, int antal, VECTFLOAT scale)
{
    int i;
    for (i = 0; i < antal; i++, source++, dest++) {
	dest->x = IMULIF(source->x, scale);
	dest->y = IMULIF(source->y, scale);
	dest->z = IMULIF(source->z, scale);
    }
}

void stretchcords(CORD3D *source, CORD3D *dest, int antal, CORD3D *scale)
{
    int i;
    for (i = 0; i < antal; i++, source++, dest++) {
	dest->x = IMULIF(source->x, scale->x);
	dest->y = IMULIF(source->y, scale->y);
	dest->z = IMULIF(source->z, scale->z);
    }
}

void transcords(CORD3D *source, CORD3D *dest, int antal, CORD3D *trans)
{
    int i;
    for (i = 0; i < antal; i++, source++, dest++) {
	dest->x = source->x + trans->x;
	dest->y = source->y + trans->y;
	dest->z = source->z + trans->z;
    }
}

void flipcords(CORD3D *source, CORD3D *dest, int antal, AXEL axel)
{
    int i;
    for (i = 0; i < antal; i++, source++, dest++) {
	switch (axel) {
	case X_AXEL:
	    dest->x = -source->x;
	    break;
	case Y_AXEL:
	    dest->y = -source->y;
	    break;
	case Z_AXEL:
	    dest->z = -source->z;
	    break;
	}
    }
}

int cmpcords(CORD3D *c1, CORD3D *c2)
{
    if (c1->x != c2->x)
	return 1;
    if (c1->y != c2->y)
	return 1;
    if (c1->z != c2->z)
	return 1;
    return 0;
}

int getsincos(int angle, short *si, short *co)
{
    extern const short sintab[];
    int sina;
    int cosa;

    while (angle < 0)
	angle += 360;
    while (angle >= 360)
	angle -= 360;
    sina = angle;
    *si = sintab[sina];
    cosa = sina + 90;
    if (cosa >= 360)
	cosa -= 360;
    *co = sintab[cosa];

    return angle;
}
